function [data]=deci(data,factor)
%DECI    Downsample SAClab data records
%
%    Description: Downsamples SAClab data records by an integer factor.  
%     Keep the factor below 13 to avoid anti-aliasing filter problems.
%     Cascades are allowed by supplying multiple factors.  Uses the Matlab
%     function decimate (Signal Processing Toolbox).
%
%    Usage:  [data]=stretch(data,factors)
%
%    Examples: 
%     to halve the samplerates
%     [data]=deci(data,2)
%
%     to cascade to a samplerate 40 times lower
%     [data]=deci(data,[5 8])
%
%    by Garrett Euler (1/2008)   ggeuler@wustl.edu
%
%    See also: stretch, syncsr, intrplt
%

% check inputs
error(nargchk(2,2,nargin))

% check data structure
if(~isstruct(data))
    error('input data is not a structure')
elseif(~isvector(data))
    error('data structure not a vector')
elseif(~isfield(data,'version') || ~isfield(data,'head') || ...
        ~isfield(data,'amp'))
    error('data structure does not have proper fields')
elseif(isfield(data,'time'))
    error('function does not work with unevenly sampled data')
end

% check factor
if(~isvector(factor) || ~isinteger(factor) || factor<1)
    error('factor must be a vector of positive integer(s)')
end

% number of factors
nf=length(factor);

% overall factor
of=prod(factor);

% header info
[b,delta]=gh(data,'b','delta');

% decimate and update header
for i=1:length(data)
    % loop over factors
    for j=1:nf
        % stretch
        data(i).amp=decimate(data(i).amp,factor(j));
    end
    
    % update header
    data(i)=ch(data(i),'npts',length(data(i).amp),'delta',delta(i)*of,...
        'e',b(i)+(length(data(i).amp)-1)*(delta(i)*of),...
        'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
        'depmax',max(data(i).amp));
end

end