function [data]=dif(data)
%DIF    Differentiates SAClab data records using discrete differences
%
%    Description: Calculates and returns the derivative of each record
%     using the differences between points as an approximation of the 
%     derivative at the midpoint.  Works with unevenly spaced data.  Uses
%     the Matlab function diff.
%
%    Usage: [data]=dif(data);
%
%    by Garrett Euler (1/2008)   ggeuler@wustl.edu
%
%    See also: integrt

% check nargin
error(nargchk(1,1,nargin))

% check data structure
if(~isstruct(data))
    error('input data is not a structure')
elseif(~isvector(data))
    error('data structure not a vector')
elseif(~isfield(data,'version') || ~isfield(data,'head') || ...
        ~isfield(data,'amp'))
    error('data structure does not have proper fields')
end

% number of seismograms
nrecs=length(data);

% retreive header info
[delta,b,e,npts,leven]=gh(data,'delta','b','e','npts','leven');

% take derivative and update header
for i=1:nrecs
    % evenly spaced?
    if (leven(i))
        data(i).amp=diff(data(i).amp)/delta(i);
        b(i)=b(i)+delta(i)/2; e(i)=e(i)-delta(i)/2; npts(i)=npts(i)-1;
    else
        times=diff(data(i).time);
        data(i).amp=diff(data(i).amp)./times;
        data(i).time=data(i).time(1:npts-1)+times/2;
        npts(i)=length(data(i).amp); b(i)=data(i).time(1);
        e(i)=data(i).time(end);
    end
    
    % update header
    data(i)=ch(data(i),'depmen',mean(data(i).amp),...
        'depmin',min(data(i).amp),'depmax',max(data(i).amp),...
        'b',b(i),'e',e(i),'npts',npts(i));
end

end