function [data]=rpdw(data,ref1,offset1,ref2,offset2,fill,filler)
%RPDW    Read partial data window from SAC binary file
%
%    Description: Reads a partial data window from a SAC (seismic analysis
%     code) binary file using the reference and offset parameters given.
%     This provides a mechanism similar to the SAC 'cut' command to limit
%     memory usage.  Input parameters are exactly the same as the SAClab 
%     'cutim' command - for details on cutting read there.
%
%    Usage: [data]=rpdw(data,ref1,offset1,ref2,offset2,fill,filler)
%
%    by Garrett Euler (1/2008)   ggeuler@wustl.edu
%
%    See also: cutim, rh, wh, ch, lh, gh, rsac, wsac, bsac, sachi, gv,
%              rdata

% input check
error(nargchk(1,7,nargin))

% check data structure
if(~isstruct(data))
    error('input data is not a structure')
elseif(~isvector(data))
    error('data structure not a vector')
elseif(~isfield(data,'version') || ~isfield(data,'head') || ...
        ~isfield(data,'name') || ~isfield(data,'endian'))
    error('data structure does not have proper fields')
end

% defaults
if(nargin<7); filler=0; end
if(nargin<6); fill=0; end
if(nargin<5); offset2=0; end
if(nargin<4); ref2='e'; end
if(nargin<3); offset1=0; end
if(nargin<2); ref1='b'; end

% empty cut parameter defaults
if(isempty(ref1)); if(isempty(offset1)); ref1='b'; else ref1='z'; end; end
if(isempty(ref2)); if(isempty(offset2)); ref2='e'; else ref2='z'; end; end
if(isempty(offset1)); offset1=0; end
if(isempty(offset2)); offset2=0; end
if(isempty(fill)); fill=0; end
if(isempty(filler)); filler=0; end

% number of seismograms
nrecs=length(data);

% cut parameter checks
if(~ischar(ref1) || ~ischar(ref2))
    error('references must be strings')
elseif(~isnumeric(offset1) || ~isnumeric(offset2) || ...
        ~isvector(offset1) || ~isvector(offset2))
    error('offsets must be a numeric vector')
elseif((length(offset1)~=1 && length(offset1)~=nrecs) || ...
        (length(offset1)~=1 && length(offset1)~=nrecs))
    error('offsets dimensions not correct')
end

% grab spacing info
[b,npts,delta,leven]=gh(data,'b','npts','delta','leven');

% expand scalar offsets
if(length(offset1)==1)
    offset1=repmat(offset1,nrecs,1);
end
if(length(offset2)==1)
    offset2=repmat(offset2,nrecs,1);
end

% window begin point
if(strcmpi(ref1,'z'))
    bt(1:nrecs,1)=offset1;
elseif(strcmpi(ref1,'n'))
    bp=round(offset1);
else
    bt=gh(data,ref1)+offset1;
end

% window end time
if(strcmpi(ref2,'z'))
    et(1:nrecs,1)=offset2;
elseif(strcmpi(ref2,'n') || strcmpi(ref2,'l'))
    ep=round(offset2);
else
    et=gh(data,ref2)+offset2;
end

% loop through each file
destroy=[];
for i=1:nrecs
    % check if evenly spaced
    if(leven(i))
        % calculate begin and end points
        if(~strcmpi(ref1,'n'))
            bp(i)=round((bt(i)-b(i))/delta(i))+1;
        end
        if(strcmpi(ref2,'l'))
            ep2=bp(i)+ep(i)-1;
        elseif(strcmpi(ref2,'n'))
            ep2=ep(i);
        else
            ep2=round((et(i)-b(i))/delta(i))+1;
        end
        
        % boundary conditions
        nbp=max([bp(i) 1]);
        nep=min([ep2 npts(i)]);
        
        % get header info
        header=sachp(data(i).version);
        
        % open file
        fid=fopen(data(i).name,'r',data(i).endian);
        
        % check that it opened
        if(fid<0)
            warning('SAClab:fileNoExist','File does not exist: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % move to first byte
        try
            fseek(fid,header.bytes+header.databytes*(nbp-1),'bof');
        catch
            warning('SAClab:seekFailed','Seek to data failed: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % read in data
        data(i).amp=fread(fid,nep-nbp+1,header.datastorage);
        
        % verify read
        if(length(data(i).amp)~=nep-nbp+1)
            warning('SAClab:readFailed','Reading data failed: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % fill or no
        if(fill)
            % add zeros
            data(i).amp=[ones(1-bp(i),1)*filler; data(i).amp; ones(ep2-npts(i),1)*filler];
            
            % empty window - add to destroy list
            if(isempty(data(i).amp))
                destroy=[destroy; i];
                continue;
            end
            
            % fix header
            data(i)=ch(data(i),'b',b(i)+(bp(i)-1)*delta(i),...
                'e',b(i)+(ep2-1)*delta(i),'npts',length(data(i).amp),...
                'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
                'depmax',max(data(i).amp));
        else
            % empty window - add to destroy list
            if(isempty(data(i).amp))
                destroy=[destroy; i];
                continue;
            end
                    
            % fix header
            data(i)=ch(data(i),'b',b(i)+(nbp-1)*delta(i),...
                'e',b(i)+(nep-1)*delta(i),'npts',length(data(i).amp),...
                'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
                'depmax',max(data(i).amp));
        end
    % unevenly spaced
    else
        % get begin point
        if(~strcmpi(ref1,'n'))
            % save to temp variable (avoids corruption in no result case)
            temp=find(data(i).time>=bt(i),1);
            if(isempty(temp))
                % empty window - add to destroy list
                destroy=[destroy; i];
                continue;
            elseif(temp>1)
                temp2=temp-1;
                
                % figure out which is closer
                if(abs(bt(i)-data(i).time(temp))<=abs(bt(i)-data(i).time(temp2)))
                    bp(i)=temp;
                else
                    bp(i)=temp2;
                end
            else
                bp(i)=temp;
            end
        end
        
        % get end point
        if(strcmpi(ref2,'l'))
            ep2=bp(i)+ep(i)-1;
        elseif(strcmpi(ref2,'n'))
            ep2=ep(i);
        else
            % save to temp variable (avoids corruption in no result case)
            temp=find(data(i).time<=et(i),1,'last');
            if(isempty(temp))
                % empty window - add to destroy list
                destroy=[destroy; i];
                continue;
            elseif(temp<npts(i))
                temp2=temp+1;
                
                % figure out which is closer
                if(abs(et(i)-data(i).time(temp))<=abs(et(i)-data(i).time(temp2)))
                    ep2=temp;
                else
                    ep2=temp2;
                end
            else
                ep2=temp;
            end
        end
        
        % boundary conditions
        nbp=max([bp(i) 1]);
        nep=min([ep2 npts(i)]);
        
        % check if bp>=ep
        if(nbp>nep)
            destroy=[destroy; i];
            continue;
        end
        
        % get header info
        header=sachp(data(i).version);
        
        % open file
        fid=fopen(data(i).name,'r',data(i).endian);
        
        % check that it opened
        if(fid<0)
            warning('SAClab:fileNoExist','File does not exist: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % move to first byte
        try
            fseek(fid,header.bytes+header.databytes*(nbp-1),'bof');
        catch
            warning('SAClab:seekFailed','Seek to data failed: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % read in data
        data(i).amp=fread(fid,nep-nbp+1,header.datastorage);
        
        % verify read
        if(length(data(i).amp)~=nep-nbp+1)
            warning('SAClab:readFailed','Reading data failed: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % move to first byte
        try
            fseek(fid,header.bytes+header.databytes*(npts(i)+nbp-1),'bof');
        catch
            warning('SAClab:seekFailed','Seek to data failed: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % read in data
        data(i).time=fread(fid,nep-nbp+1,header.datastorage);
        
        % verify read
        if(length(data(i).time)~=nep-nbp+1)
            warning('SAClab:readFailed','Reading data failed: %s',data(i).name);
            destroy=[destroy; i];
            continue;
        end
        
        % fix header
        if(nbp==nep)
            data(i)=ch(data(i),'b',data(i).time(1),...
                'e',data(i).time(end),'npts',length(data(i).amp),...
                'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
                'depmax',max(data(i).amp),...
                'delta',0,'odelta',0);
        else
            data(i)=ch(data(i),'b',data(i).time(1),...
                'e',data(i).time(end),'npts',length(data(i).amp),...
                'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
                'depmax',max(data(i).amp),...
                'delta',(data(i).time(end)-data(i).time(1))/(npts(i)-1),...
                'odelta',data(i).time(min([end 2]))-data(i).time(1));
        end
    end
end

% destroy empty records
data(destroy)=[];

end
