function [data]=stretch(data,factor)
%STRETCH    Upsample SAClab data records
%
%    Description: Upsamples SAClab data records by an integer factor.  
%     Anti-aliasing is not an issue so there is no limit imposed on the
%     stretch factor.  Cascades are allowed regardless.  Uses the Matlab
%     function interp (Signal Processing Toolbox).
%
%    Usage:  [data]=stretch(data,factor)
%
%    Examples: 
%       to double the samplerates
%       [data]=stretch(data,2)
%
%       to cascade to a samplerate 40 times higher
%       [data]=stretch(data,[5 8])
%
%    by Garrett Euler (1/2008)   ggeuler@wustl.edu
%
%    See also: deci, syncsr, intrplt
%

% check inputs
error(nargchk(2,2,nargin))

% check data structure
if(~isstruct(data))
    error('input data is not a structure')
elseif(~isvector(data))
    error('data structure not a vector')
elseif(~isfield(data,'version') || ~isfield(data,'head') || ...
        ~isfield(data,'amp'))
    error('data structure does not have proper fields')
elseif(isfield(data,'time'))
    error('function does not work with unevenly sampled data')
end

% check factor
if(~isvector(factor) || ~isinteger(factor) || factor<1)
    error('factor must be a vector of positive integer(s)')
end

% number of factors
nf=length(factor);

% overall factor
of=prod(factor);

% header info
delta=gh(data,'delta');

% stretch and update header
for i=1:length(data)
    % loop over factors
    for j=1:nf
        % stretch
        data(i).amp=interp(data(i).amp,factor(j),4,1);
    end
    
    % update header
    data(i)=ch(data(i),'npts',length(data(i).amp),'delta',delta(i)/of,...
        'depmen',mean(data(i).amp),'depmin',min(data(i).amp),...
        'depmax',max(data(i).amp));
end

end